local function bossThinker()
	server.boss = $ or nil
	
	local playerCount = 0
	local possiblePlayers = {}
	for p in players.iterate
		if (p and p.mo) and not p.spectator
			table.insert(possiblePlayers, p)
			playerCount = $+1
		end
	end

	if server.gamemode == GM_BOSS and (leveltime == 5*TICRATE) then			
		server.boss = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
		server.boss.isboss = true
		
		print("The boss is "..server.boss.name.."! Defeat them!")		
		for p in players.iterate do
			if p.spectator then continue end

			if p.isboss then
				p.hp = 70*playerCount
				p.maxhp = 70*playerCount
			else
				p.hp = 70
				p.maxhp = 70
			end
			
			p.ultimate = 0
		end
	end
	
	for p in players.iterate do
		if p.spectator then continue end

		if p.isboss then
			p.mo.destscale = (FRACUNIT/2)
			p.mo.colorized = true
		end
		
		p.ultimate = $ or 0
	end
	
	if (leveltime == 8*TICRATE) then 
		S_ChangeMusic("KTBOSS")
		showMusicCredit()
	end

	local spark, laser, ghost
	for p in players.iterate do
		if p.spectator then continue end

		if p.isboss then
			p.mo.scale = 2*FRACUNIT
			p.kartstuff[k_boostpower] = 2*FRACUNIT

			spark = P_SpawnMobj(p.mo.x + P_RandomRange(-64, 64)*FRACUNIT, p.mo.y + P_RandomRange(-64, 64)*FRACUNIT, p.mo.z + P_RandomRange(0, 10)*FRACUNIT, MT_SUPERSPARK)
			spark.color = p.skincolor
			spark.colorized = true
			spark.tics = 35
			spark.fuse = 35
			spark.scale = FRACUNIT/2
			spark.destscale = 1
			spark.momz = P_RandomRange(1, 5)*FRACUNIT/2
		end

		if p.hitcool then
			p.hitcool = $-1
			if p.hitcool <= 0 then p.hitcool = nil end
		end

		if p.ultimate and p.ultimate >= 100 and p.cmd.buttons & BT_BRAKE and not p.laser and not p.leaping and not p.down then
			if p.isboss
				p.laser = 10*TICRATE
			else
				p.leaping = 15
			end
			
			p.ultimate = 0
		end
		
		if p.isboss then
			if p.laser then
				p.mo.momx = 0
				p.mo.momy = 0
				p.mo.momz = 0
				p.mo.shootangle = $ or p.mo.angle

				/*
				if (p.cmd.buttons & BT_TURNLEFT)
					p.mo.shootangle = $-ANG1
				elseif (p.cmd.buttons & BT_TURNRIGHT)
					p.mo.shootangle = $+ANG1
				end
				*/

				if not (leveltime%10) then
					for i = -6, 6 do
						laser = P_SPMAngle(p.mo, MT_LASERKBOSS, p.mo.shootangle+(i*ANG10))
						laser.angle = p.mo.shootangle+(i*ANG10)
					end
				end
				
				p.laser = $-1
				if p.laser <= 0
					p.laser = nil
					p.mo.shootangle = nil
				end
			end
		else
			if p.leaping 
				P_SpawnGhostMobj(p.mo)
				P_SetObjectMomZ(p.mo, 20*FRACUNIT)
				p.leaping = $-1

				if p.leaping <= 0 
					p.down = 4 
					p.leaping = nil
				end
			elseif p.down
				ghost = P_SpawnGhostMobj(p.mo)
				ghost.destscale = 5*FRACUNIT

				P_SetObjectMomZ(p.mo, -40*FRACUNIT)
				
				if P_IsObjectOnGround(p.mo) and not (leveltime%15)
					K_SpawnMineExplosion(p.mo, p.mo.color)
					K_SpawnKartExplosion(p.mo.x, p.mo.y, p.mo.z, 800*FRACUNIT, nil, nil, nil, nil, nil, p.mo)
					
					p.down = $-1
					if p.down <= 0 then p.down = nil end
				end
			end
		end
	end

	if not (leveltime%350)
		local possiblePlayers = {}

		for p in players.iterate
			if p.spectator then continue end

			if validP(p) then table.insert(possiblePlayers, p.mo) end
		end

		if #possiblePlayers then
			local tmo = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
			local item = P_SpawnMobj(tmo.x+P_RandomRange(-3000, 3000)<<FRACBITS, tmo.y+P_RandomRange(-3000, 3000)<<FRACBITS, tmo.ceilingz-P_RandomRange(0, 150)<<FRACBITS, MT_RANDOMITEM)
			item.flags = $ & ~MF_NOGRAVITY & ~MF_NOCLIPHEIGHT & ~MF_SLIDEME
		end
	end

	-- End Terms
	local playersLeft = {}
	for p in players.iterate
		if p.boxdata.out == false and not p.isboss and not p.spectator then
			table.insert(playersLeft, p)
		end
	end
	
	if server.boss and server.boss.valid
		if server.boss.boxdata.out == true
			for i = 1, #playersLeft
				playersLeft[i].marescore = 1
			end

			server.boss.marescore = 0
			print("The Boss was defeated!")

			BX_EndBattle()
		elseif #playersLeft <= 0
			server.boss.marescore = 1
			
			for i = 1, #playersLeft
				playersLeft[i].marescore = 0
			end
			
			print("The Players were defeated!")
			
			BX_EndBattle()
		end
	end
end

local function bossHud(v, p, c)
	if server and server.valid
		if server.boss and server.boss.mo and server.boss.mo.valid
			local b = server.boss

			if b.hp
				local BOSSBARLENGTH = 160

				v.drawString(76, 195, b.name..":"..b.hp.."/"..b.maxhp.."HP", V_SNAPTOBOTTOM|V_SNAPTORIGHT, "small-right")
				v.drawFill(80, 195, BOSSBARLENGTH, 2, V_SNAPTOBOTTOM|V_SNAPTORIGHT|31)
				v.drawFill(80, 195, b.hp*BOSSBARLENGTH/b.maxhp+(1/2), 2, V_SNAPTOBOTTOM|V_SNAPTORIGHT|126)

				v.drawFill(80, 191, BOSSBARLENGTH, 2, V_SNAPTOBOTTOM|V_SNAPTORIGHT|31)
				v.drawFill(80, 191, b.ultimate*BOSSBARLENGTH/100+(1/2), 2, V_SNAPTOBOTTOM|V_SNAPTORIGHT|205)

				if b.ultimate >= 100
					v.drawString(110, 185, "ULTIMATE", V_SNAPTOTOP|V_SNAPTOLEFT)
				end
			end
			
			if p.hp and not p.isboss
				local PBARLENGTH = 60

				v.drawString(0, 70, p.hp.."/"..p.maxhp.."HP", V_SNAPTOTOP|V_SNAPTOLEFT)
				v.drawFill(0, 60, PBARLENGTH, 2, V_SNAPTOTOP|V_SNAPTOLEFT|31)
				v.drawFill(0, 60, p.hp*PBARLENGTH/p.maxhp+(1/2), 2, V_SNAPTOTOP|V_SNAPTOLEFT|39)
				
				v.drawFill(0, 56, PBARLENGTH, 2, V_SNAPTOBOTTOM|V_SNAPTORIGHT|31)
				v.drawFill(0, 56, p.ultimate*PBARLENGTH/100+(1/2), 2, V_SNAPTOBOTTOM|V_SNAPTORIGHT|205)
				
				if p.ultimate >= 100
					v.drawString(0, 80, "READY! BRAKE!", V_SNAPTOTOP|V_SNAPTOLEFT)
				end
			end
		end
	end
end

freeslot("sfx_bossdm")
local function bossPain(p, inf, src)
	if p.hitcool then return false end

	if (p and p.mo)		
		local dmg = P_RandomRange(3, 10)
		if (src and src.player and p ~= src.player)	
			if not p.isboss and not src.player.isboss
				K_DoInstashield(p)
				return false
			end

			src.player.ultimate = min(100, $+(dmg*5)/2)
		elseif (inf and inf.player and p ~= inf.player)
			if not p.isboss and not inf.player.isboss
				K_DoInstashield(p)
				return false
			end

			inf.player.ultimate = min(100, $+(dmg*5)/2)
		end

		p.hp = max(0, p.isboss and $-(dmg*3) or $-dmg)
		if p.hp <= 0
			p.boxdata.out = true
			p.boxdata.survivaltime = leveltime-5*TICRATE
			BX_ExplodeAndRespawn(p)
			return false
		else
			p.hitcool = 35
		end
		
		p.kartstuff[k_bumper] = 10
		S_StartSound(p.mo, sfx_bossdm)
		
		P_InstaThrust(p.mo, R_PointToAngle2(inf.x, inf.y, p.mo.x, p.mo.y), 30*FRACUNIT)
		p.kartstuff[k_spinouttimer] = TICRATE/2
		return false
	end
end

local function bossReset()
	server.boss = nil
	
	for p in players.iterate
		p.isboss = nil
		p.hitcool = nil
		
		p.laser = nil
		p.leaping = nil
		p.down = nil
	end
end

Box.Gamemodes[GM_BOSS] = {
	name = "Bossfight",
	hint = "Take down the opposition!",
	noBumpers = true,
	thinker = bossThinker,
	shouldspin = bossPain,
	shouldsquish = bossPain,
	shouldexplode = bossPain,
	resetfunc = bossReset,
	hudfunc = bossHud
}