local LB_GREY = 0
local LB_RED = 1
local LB_BLUE = 2
local LB_GREEN = 3
local LB_YELLOW = 4
local textures = {"", "REDNOISE", "BLUNOISE", "GRNNOISE", "YELNOISE"}
local skinColors = {0, 128, 230, 184, 104}

local function setColors()
	server.colors = {}
	for sector in sectors.iterate
		server.colors[#sector] = P_RandomRange(2, 5)
		sector.floorpic = textures[server.colors[#sector]]
	end
end

local function chooseColor()			
	for p in players.iterate
		if validP(p)
			p.kartstuff[k_itemtype] = 4
			p.kartstuff[k_itemamount] = 1
		end
	end

	server.targColor = P_RandomRange(2, 5)
	server.selColor = 105
end

local function restoreSectors()
	for sector in sectors.iterate
		sector.floorheight = 0
		sector.floorpic = textures[server.colors[#sector]]
	end
end

local function lavaBox()
	server.removesectors = $ or {}
	
	if (leveltime == 2) then setColors() end
	if not (leveltime%450) and (leveltime > 6*TICRATE) then chooseColor() end
	
	if server.selColor
		if not server.skinColorPick
			server.skinColorPick = 2
		end

		if not (leveltime%5)
			server.skinColorPick = $+1
			if server.skinColorPick > 5
				server.skinColorPick = 2
			end

			for p in players.iterate
				if validP(p)
					local sound = {sfx_itrol1, sfx_itrol2, sfx_itrol3, sfx_itrol4, sfx_itrol5, sfx_itrol6, sfx_itrol7, sfx_itrol8}
					S_StartSound(nil, sound[P_RandomRange(1, #sound)], p)
				end
			end
		end

		server.selColor = $-1
		if server.selColor <= 0
			server.selColor = nil
			server.colorTime = 140
		end
	end
	
	if server.colorTime
		server.colorTime = $-1
		if server.colorTime <= 0
			server.colorTime = nil
			for s in sectors.iterate
				if server.colors[#s] ~= server.targColor
					s.floorheight = -300*FRACUNIT
					s.floorpic = "LAVAR1_1"
				end
			end
			
			server.restoreTime = 220
		end
	end
	
	if server.restoreTime
		server.restoreTime = $-1
		if server.restoreTime <= 0
			server.restoreTime = nil
			restoreSectors()
		end
	end
	
	for p in players.iterate
		if p.spectator then continue end

		if p.mo.subsector.sector.floorpic == "LAVAR1_1" 
		and p.boxdata and p.boxdata.out == false
		and p.mo.z <= p.mo.subsector.sector.floorheight
			p.boxdata.out = true
			p.boxdata.survivaltime = G_TicsToSeconds(leveltime-5*TICRATE)
			BX_ExplodeAndRespawn(p)

			print(p.name.." fell into a pit.")
		end
	end
	
	-- End Terms
	local playersLeft = {}
	for p in players.iterate
		if p.boxdata.out == false
			table.insert(playersLeft, p)
		end
	end
	
	server.ended = $ or false
	if #playersLeft <= 1 and server.ended == false
		for p in players.iterate
			if p.boxdata.out == true
				p.marescore = p.boxdata.survivaltime or 0
			else
				p.marescore = 69420
			end
		end
		
		BX_EndBattle()
		server.ended = true
	end
	
	if (leveltime == 8*TICRATE) then
		S_ChangeMusic("CRUMBL")
		showMusicCredit()
	end
end

local function lavaPush(p, inf, src)
	if p.mo and p.mo.valid and inf and inf.valid
		local ang = R_PointToAngle2(inf.x, inf.y, p.mo.x, p.mo.y)
		P_Thrust(p.mo, ang, 50*FRACUNIT)
	end
	
	return true
end

local function lavaReset()
	server.colors = nil
	server.selColor = nil
	server.colorTime = nil
	server.restoreTime = nil
	server.removesectors = nil
	server.skinColorPick = nil
end

local function lavaHud(v, p, c)
	if validP(p)
		if server.selColor
			v.draw(120, 5, v.cachePatch("K_ITBG"), V_SNAPTOTOP|V_SNAPTOLEFT)
			v.drawFill(130, 15, 30, 30, V_SNAPTOTOP|V_SNAPTOLEFT|skinColors[server.skinColorPick])
		end

		if server.colorTime
			v.draw(120, 5, v.cachePatch("K_ITBG"), V_SNAPTOTOP|V_SNAPTOLEFT)
			v.drawFill(130, 15, 30, 30, V_SNAPTOTOP|V_SNAPTOLEFT|skinColors[server.targColor])
			v.drawString(170, 30, "GO TO THIS COLOR", V_SNAPTOTOP|V_SNAPTOLEFT, "center")
		end
	end
end
	
Box.Gamemodes[GM_LAVABOX] = {
	name = "Lava Box",
	hint = "Be the last one standing!",
	noBumpers = true,

	thinker = lavaBox,
	hudfunc = lavaHud,
	spinoutfunc = lavaPush,
	squishfunc = lavaPush,
	explodefunc = lavaPush,
	resetfunc = lavaReset,
}