local function railRingLoad()
	for s in sectors.iterate
		s.ceilingheight = 300*FRACUNIT
	end

	server.ignoreX = {}
	server.ignoreY = {}
	for x = 1, MAP_WIDTH
		for y = 1, MAP_HEIGHT
			local s = R_PointInSubsector(x*SEC_LENGTH, y*SEC_LENGTH).sector
			s.floorheight = 300*FRACUNIT
			for p in players.iterate
				if validP(p) and p.mo.subsector.sector == s
					table.insert(server.ignoreX, x)
					table.insert(server.ignoreY, y)
				end
			end
		end
	end
	
	-- now we have server.ignore tables...
	if #server.ignoreX and #server.ignoreY
		for x = 1, MAP_WIDTH
			for y = 1, MAP_HEIGHT
				for i = 1, #server.ignoreX
					local s = R_PointInSubsector(x*SEC_LENGTH, y*SEC_LENGTH).sector
					if server.ignoreX[i] == x
						s.floorheight = 0
					end
				end

				for i = 1, #server.ignoreY
					local s = R_PointInSubsector((x*SEC_LENGTH) - (SEC_LENGTH/2), (y*SEC_LENGTH) + (SEC_LENGTH/2)).sector
					if server.ignoreY[i] == y
						s.floorheight = 0
					end
				end
			end
		end
	end
end

local function railRingPThink(p)
	p.mo.railCool = $ or 0
	if p.mo.railCool > 0 then p.mo.railCool = $-1 end
	
	if P_PlayerInPain(p) or (p.kartstuff[k_bumper] <= 0) then return end
	
	p.mo.buffs = $ or {
		speed = 0,
		range = 0,
		cooldown = 0,
	}
	
	for k,v in ipairs(p.mo.buffs)
		if v > 3 then v = 3 end
	end

	if p.mo and p.mo.valid and (p.cmd.buttons & BT_ATTACK) and not p.mo.railCool
		K_PlayAttackTaunt(p.mo)
		p.mo.railCool = 2*TICRATE-(p.mo.buffs.cooldown*(TICRATE/3))

		if p.mo.buffs.range <= 0
			local missile = P_SPMAngle(p.mo, MT_ROCKET, p.mo.angle, 0, MF2_RAILRING)
			
			if missile and missile.valid
				P_InstaThrust(missile, p.mo.angle, 115*FRACUNIT+(p.mo.buffs.speed*15*FRACUNIT))
			end
		else
			for i = -p.mo.buffs.range, p.mo.buffs.range
				local ang = p.mo.angle + i*ANG20
				local missile = P_SPMAngle(p.mo, MT_ROCKET, ang, 0, MF2_RAILRING)
				
				if missile and missile.valid
					P_InstaThrust(missile, ang, 115*FRACUNIT+(p.mo.buffs.speed*15*FRACUNIT))
				end
			end
		end
	end
end

local function railRing()
	if (leveltime < 8*TICRATE) then return end

	local possiblePlayers = {}
	for p in players.iterate do
		if validP(p) then
			railRingPThink(p)
			table.insert(possiblePlayers, p.mo)
		end
	end

	if not (leveltime%60) then
		if #possiblePlayers then
			local tmo = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
			local item = P_SpawnMobj(tmo.x+P_RandomRange(-128, 128)*FRACUNIT, tmo.y+P_RandomRange(-128, 128)*FRACUNIT, tmo.z, randBuff())
			item.scale = 1
			item.destscale = FRACUNIT
		end
	end

	if (leveltime == 8*TICRATE) then
		S_ChangeMusic("MARCH")
		showMusicCredit()
	end
end

local function railRingDmg(p, inf, src)
	if p.mo and p.mo.valid and p.mo.buffs
		for k,v in pairs(p.mo.buffs)
			if v >= 0 then v = $-1 end
		end
	end
end

local function railRingHUD(v, p, c)
	if validP(p) and p.mo.buffs
		v.drawString(90, 182, "SPD: "..p.mo.buffs.speed, V_SNAPTOLEFT|V_SNAPTOBOTTOM)
		v.drawString(145, 182, "RNG: "..p.mo.buffs.range, V_SNAPTOLEFT|V_SNAPTOBOTTOM)
		v.drawString(200, 182, "COOL: "..p.mo.buffs.cooldown, V_SNAPTOLEFT|V_SNAPTOBOTTOM)
	end
end

Box.Gamemodes[GM_RAILRING] = {
	name = "Shooter",
	hint = "Shoot everyone down! Be the last one!",
	bumpers = 5,
	loadFunc = railRingLoad,
	thinker = railRing,
	damagefunc = railRingDmg,
	spinfunc = railRingDmg,
	squishfunc = railRingDmg,
	explodefunc = railRingDmg,
	hudfunc = railRingHUD
}