local function sectorClaim()
	if not server.sectorcolors
		server.sectorcolors = {}

		for sector in sectors.iterate
			server.sectorcolors[#sector] = "gray"
		end
	end

	if (leveltime == 8*TICRATE)
		server.sectorclaimtimer = 180*TICRATE
		S_ChangeMusic("RANDOM")
		showMusicCredit()
	end
	
	if not (leveltime%850)
		local possiblePlayers = {}

		for p in players.iterate
			if validP(p) then table.insert(possiblePlayers, p.mo) end
		end

		if #possiblePlayers
			local tmo = possiblePlayers[P_RandomRange(1, #possiblePlayers)]
			local item = P_SpawnMobj(tmo.x+P_RandomRange(-3000, 3000)<<FRACBITS, tmo.y+P_RandomRange(-3000, 3000)<<FRACBITS, tmo.ceilingz-P_RandomRange(0, 150)<<FRACBITS, MT_RANDOMITEM)
			item.flags = $ & ~MF_NOGRAVITY & ~MF_NOCLIPHEIGHT & ~MF_SLIDEME
		end
	end
	
	if server.sectorclaimtimer
		server.sectorclaimtimer = $-1
		
		if server.sectorclaimtimer <= 0
			local teamPoints = {
				["red"] = 0,
				["blue"] = 0,
				["green"] = 0,
				["yellow"] = 0,
			}

			for sector in sectors.iterate
				if server.sectorcolors[#sector] ~= "gray"
					teamPoints[server.sectorcolors[#sector]] = $+1
				end
			end
			
			for p in players.iterate
				if not p.spectator and p.team
					p.marescore = teamPoints[p.team] or 0
					p.mo.color = p.skincolor
				end
			end
			
			BX_EndBattle()
		end
	end
end

local function sectorPlayerThink(p)
	if not p.claimedonentry
		p.claimedonentry = {}
		for s in sectors.iterate
			p.claimedonentry[#s] = false
		end
	end
	
	if p.mo.subsector.sector
		local sector = p.mo.subsector.sector
		
		if P_IsObjectOnGround(p.mo) and p.claimedonentry[#sector] == false
		and not p.kartstuff[k_spinouttimer]
			p.claimedonentry[#sector] = true
			S_StartSound(p.mo, sfx_ghit)
			
			if p.team == "red"
				sector.floorpic = "REDNOISE"
			elseif p.team == "blue"
				sector.floorpic = "BLUNOISE"
			elseif p.team == "green"
				sector.floorpic = "GRNNOISE"
			elseif p.team == "yellow"
				sector.floorpic = "YELNOISE"
			end
			
			server.sectorcolors[#sector] = p.team
		end
	end
	
	if p.claimedonentry
		for s in sectors.iterate
			if s ~= p.mo.subsector.sector
				p.claimedonentry[#s] = false
			end
		end
	end
	
	if p.kartstuff[k_spinouttimer] and not p.paincheck
		local ghost = P_SpawnGhostMobj(p.mo)
		ghost.destscale = 5*FRACUNIT

		P_InstaThrust(p.mo, p.mo.stunangle or p.mo.angle+ANGLE_180, 100*FRACUNIT)
		p.kartstuff[k_spinouttimer] = 69
		
		if (p.mo.eflags & MFE_JUSTBOUNCEDWALL)
			p.mo.stunangle = nil
			p.kartstuff[k_spinouttimer] = 5*TICRATE
			p.paincheck = 10*TICRATE
		end
	end
	
	if p.paincheck then p.paincheck = $-1 end
end

local function sectorPush(p, inf, src)
	if p.mo and p.mo.valid
		local ang = p.mo.angle+ANGLE_180
		if inf and inf.valid 
			ang = R_PointToAngle2(inf.x, inf.y, p.mo.x, p.mo.y)
		end

		p.mo.stunangle = ang
		p.kartstuff[k_spinouttimer] = 69 -- nice
		S_StartSound(p.mo, sfx_pstop)
	end
	
	return false
end

local function sectorReset()
	server.sectorcolors = nil
	server.sectorclaimtimer = nil
end

local function sectorHUD(v, p, c)
	if server and server.sectorclaimtimer and p and p.valid and not p.spectator
		local t = server.sectorclaimtimer
		
		v.drawString(160, 0, G_TicsToMinutes(t)..":"..G_TicsToSeconds(t).." left", V_SNAPTOTOP|V_SNAPTOLEFT, "center")
	end
end

Box.Gamemodes[GM_SECTORCLAIM] = {
	name = "Sector Claim",
	hint = "Claim the most sectors with your team!",
	noBumpers = true,
	teams = 2,

	hudfunc = sectorHUD,
	thinker = sectorClaim,
	playerthinker = sectorPlayerThink,
	spinoutfunc = sectorPush,
	squishfunc = sectorPush,
	explodefunc = sectorPush,
	resetfunc = sectorReset,
}