rawset(_G, "Box", {
	Gamemodes = {},
	SimonSays = {},
	
	Teams = {
		["red"] = {},
		["blue"] = {},
		["green"] = {},
		["yellow"] = {}
	},
})

// Map Defs
rawset(_G, "MAP_WIDTH", 22)
rawset(_G, "MAP_HEIGHT", 22)
rawset(_G, "SEC_LENGTH", 512*FRACUNIT)

rawset(_G, "getSecFromCoords", function(x, y)
	return R_PointInSubsector(x*SEC_LENGTH-FRACUNIT, y*SEC_LENGTH-FRACUNIT).sector
end)

// Gamemode Defs
rawset(_G, "GM_CRUSHER", 1) -- last to die wins
rawset(_G, "GM_RANDOMMOVEMENT", 2) -- terrain moves randomly every minute
rawset(_G, "GM_BOSS", 3) -- one boss is selected and must be defeated
rawset(_G, "GM_BOMBRAIN", 4) -- raining bombs. last to get hit wins
rawset(_G, "GM_CRUMBLE", 5) -- raining bombs. last to get hit wins
rawset(_G, "GM_SECTORCLAIM", 6) -- claim sectors
rawset(_G, "GM_MOVEORDIE", 7) -- self explanitory
rawset(_G, "GM_SIMONSAYS", 8) -- simonsays
rawset(_G, "GM_RAILRING", 9) -- fps esque stuff
rawset(_G, "GM_SINKSTRIKE", 10) -- sinks
rawset(_G, "GM_HELLRAIN", 11) -- rain ALL items and enemies too
rawset(_G, "GM_LAVABOX", 12) -- go to the specified color
rawset(_G, "GM_TAG", 13) -- tag.
rawset(_G, "GM_RINGRUSH", 14) -- ring rush
rawset(_G, "GM_SHRINK", 15) -- the platform gradually gets smaller
rawset(_G, "GM_LIGHT", 16) -- get as much light as possible
rawset(_G, "GM_COMPACTOR", 17) -- smash compactor from mario party 9

-- Set the Gamemode
rawset(_G, "setGameMode", function()
	if server and server.valid
		if cv_forcemode.value
			server.gamemode = cv_forcemode.value
		else
			server.gamemode = P_RandomRange(1, #Box.Gamemodes)
		end

		local mode = Box.Gamemodes[server.gamemode] or {}
		print("Gamemode is "..(mode.name or "???").."!")

		if mode.bumpers and not mode.noBumpers
			for p in players.iterate
				if p.spectator then continue end

				p.kartstuff[k_bumper] = mode.bumpers or 3
			end
		end

		Box.Teams = {
			["red"] = {},
			["blue"] = {},
			["green"] = {},
			["yellow"] = {}
		}

		if mode.teams then
			for p in players.iterate do
				local teamCounts = {{#Box.Teams["red"], "red"}, {#Box.Teams["blue"], "blue"}}
				
				if mode.teams == 3 then
					table.insert(teamCounts, {#Box.Teams["green"], "green"})
				elseif mode.teams >= 4 then
					table.insert(teamCounts, {#Box.Teams["green"], "green"})
					table.insert(teamCounts, {#Box.Teams["yellow"], "yellow"})
				end
				
				table.sort(teamCounts, function(a, b) return a[1] > b[1] end)
				
				if teamCounts[1] == 0 then
					local teamColors = {"red", "blue", "green", "yellow"}
					local teamColor = teamColors[P_RandomRange(1, 4)]

					table.insert(Box.Teams[teamColor], p)
					p.team = teamColor
				else
					table.insert(Box.Teams[teamCounts[#teamCounts][2]], p)
					p.team = teamCounts[#teamCounts][2]
				end
			end
		else
			for p in players.iterate
				p.team = nil
			end
		end
		
		if mode.loadFunc then mode.loadFunc() end
	end
end)

-- Time Out Explosions
rawset(_G, "BX_ExplodeAndRespawn", function(p)
	if not (p and p.mo) or p.spectator then return end

	p.lives = 0
	p.pflags = $|PF_TIMEOVER
	P_DamageMobj(p.mo, nil, nil, 10000)
	
	-- Respawn
	p.pflags = $ & ~PF_TIMEOVER
	p.lives = 3
end)

-- Ends a Battle Mode Battle
rawset(_G, "BX_EndBattle", function()
	if not G_BattleGametype() return end
	
	-- End the Gamemode
	if server.gamemode and Box.Gamemodes[server.gamemode] and Box.Gamemodes[server.gamemode].resetfunc
		Box.Gamemodes[server.gamemode].resetfunc()
	end

	server.gamemode = nil

	for p in players.iterate
		if p.exiting then continue end
		
		p.exiting = 281
		K_KartUpdatePosition(p)
	end
	
	for p in players.iterate
        if not (p and p.mo) or p.spectator then continue end
		
		if p.exiting
			if p.kartstuff[k_position] == 1
				S_ChangeMusic("kbwin", false, p)
			else
				if K_IsPlayerLosing(p)
					S_ChangeMusic("kblose", false, p)
				else
					S_ChangeMusic("kbok", false, p)
				end
			end
		end
	end
end)

-- Check if we're valid.
rawset(_G, "validP", function(p)
	if p and p.valid
	and p.mo and p.mo.valid
	and not p.spectator
	and p.boxdata.out == false 
		return true
	end
	
	return false
end)

-- Check if the subsector exists.
rawset(_G, "coordsToSubsector", function(x, y)
	return R_PointInSubsector(x*SEC_LENGTH, y*SEC_LENGTH)
end)

-- Show music credit for ALL players
rawset(_G, "showMusicCredit", function()
	for p in players.iterate do
		S_ShowMusicCredit(p)
	end
end)